devices_data = [
  {
    "id": 30000,
    "criterionId": 30000,
    "name": 'Ordinateur',
    "type": 'PLATFORM',
    "criterionType": 'PLATFORM',
    "isTargeted": True,
    "isExcluded": False
  },
  {
    "id": 30001,
    "criterionId": 30001,
    "name": 'Smartphone',
    "type": 'PLATFORM',
    "criterionType": 'PLATFORM',
    "isTargeted": True,
    "isExcluded": False
  },
  {
    "id": 30002,
    "criterionId": 30002,
    "name": 'Tablette',
    "type": 'PLATFORM',
    "criterionType": 'PLATFORM',
    "isTargeted": True,
    "isExcluded": False
  },
  {
    "id": 30004,
    "criterionId": 30004,
    "name": 'Smart Tv',
    "type": 'PLATFORM',
    "criterionType": 'PLATFORM',
    "isTargeted": True,
    "isExcluded": False
  }
]